const {app, BrowserWindow, dialog, globalShortcut} = require('electron');
const path = require('path');
const url = require('url');
const fs = require('fs');
const stream = require('stream');
// var http = require('http');
// var express = require('express');
// var httpapp = express();
// httpapp.use("/", express.static(__dirname + '/local'));
const ipc = require('electron').ipcMain;
// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
let win;
const nativeImage = require('electron').nativeImage;
const image = nativeImage.createFromPath(path.join(__dirname, 'local/favicon-xj.ico'));
app.commandLine.appendSwitch('in-process-gpu')

function createWindow() {
    // Create the browser window.
    win = new BrowserWindow({
        height: 720,
        titleBarStyle: 'hidden',
        autoHideMenuBar: true,
        useContentSize: true,
        resizable: false,
        hasShadow: false,
        width: 1280,
        webPreferences: {webSecurity: false},
        transparent: false,
        frame: true,
        icon: path.join(__dirname, 'local/favicon-xj.ico')
    });
    win.loadURL(url.format({
        pathname: path.join(__dirname, 'local/loading.html'),
        protocol: 'file',
        slashes: true
    }));
    // http.createServer(httpapp).listen('12563', function() {
    //   win.loadURL('http://localhost:12563/index.html?gid=655&local=local');
    // });
    // Open the DevTools.
    // win.webContents.openDevTools();
    //win.setIgnoreMouseEvents(true);
    // Emitted when the window is closed.
    win.on('closed', () => {
        // Dereference the window object, usually you would store windows
        // in an array if your app supports multi windows, this is the time
        // when you should delete the corresponding element.
        win = null
    });

    // ipc.on('exit', () => {
    //   app.quit()
    // })
    ipc.on('window-close', function (event) {
        dialog.showMessageBox({
            'type': 'info',
            'message': '确认是否关闭窗口？',
            'icon': image,
            'buttons': ['yes', 'no']
        }, function (index) {
            event.sender.send('information-dialog-selection', index)
        })
        // app.quit()
    });
    ipc.on('asynchronous-message', (event, arg) => {
        console.log("main", arg)
    });
    ipc.on('into', () => {
        //   let rand = Math.random().toString().slice(3,8);
        //  console.log(greenworks.getSteamId())
        //   win.loadURL(`http://galgame-client.iqing.com/index.html?gid=23&apitest=1&from=iqing&platform=win&rand&steamid=${greenworks.getSteamId().steamId}`)
    });


    ipc.on('save_data', function (event, arg) {
        const data = JSON.stringify(arg);
        const ws = fs.createWriteStream(path.join(__dirname, `save/${arg.id}.save`));
        ws.write(data, 'UTF8');
        ws.end();
        event.sender.send('save_data_end');

        if (arg.id !== 'auto') {
            const autoWs = fs.createWriteStream(path.join(__dirname, `save/auto.save`));
            autoWs.write(data, 'UTF8');
            autoWs.end();
        }
    });

    ipc.on('remove_data', function (event, arg) {
        const url = path.join(__dirname, `save/${arg.id}.save`);
        if (fs.existsSync(url)) {
            fs.unlink(url, (err) => {
                if (err) {
                    event.sender.send('remove_data_end', false, url);
                    throw err;
                }
                event.sender.send('remove_data_end', true, url);
            })
        } else {
            event.sender.send('remove_data_end', false, url);
        }
    });

    ipc.on('update_data', function (event, arg) {
        console.time('update');
        if (arg instanceof Object && arg.id) {
            const url = path.join(__dirname, `save/${arg.id}.save`);
            if (fs.existsSync(url)) {
                fs.readFile(url, (err, data) => {
                    if (err) throw err;
                    const updateKeys = Object.keys(arg);
                    const ws = fs.createWriteStream(url);
                    data = JSON.parse(data.toString());
                    data = JSON.stringify(data, (k, v) => updateKeys.includes(k) ? arg[k] : v);
                    console.log(data);
                    ws.write(data, 'UTF8');
                    ws.end();
                    event.sender.send('update_data_end', true, `更新 ${url} 文件成功  \n` +
                        Object.entries(arg).map(([k, v]) => `${k} => ${v}`).join('\n')
                    );
                    console.timeEnd('update')
                })
            } else {
                event.sender.send('update_data_end', false, `${url} 文件不存在`);
            }
        } else {
            event.sender.send('update_data_end', false, 'update_data 的传入参数不是一个对象或者是其中没有有效的 id');
        }
    });

    ipc.on('appexit', function (event, arg) {
        app.exit();
    });

    ipc.on('fullscreen', function (event, arg) {
        win.setFullScreen(true);
    });

    ipc.on('exitfull', function (event, arg) {
        win.setFullScreen(false);
        // win.setSize(1280,720);
    })

    ipc.on('windev', function () {
        // win.webContents.openDevTools();
    })
}


// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.setPath('userData', app.getAppPath() + '/userData')
app.on('ready', createWindow)
// Quit when all windows are closed.
app.on('window-all-closed', () => {
    // On macOS it is common for applications and their menu bar
    // to stay active until the user quits explicitly with Cmd + Q
    if (process.platform !== 'darwin') {
        app.quit()
    }
})

app.on('activate', () => {
    // On macOS it's common to re-create a window in the app when the
    // dock icon is clicked and there are no other windows open.
    if (win === null) {
        createWindow()
    }
})


// In this file you can include the rest of your app's specific main process
// code. You can also put them in separate files and require them here.
